<?php

namespace App\Http\Controllers\AccessPublic;

use App\Campaign;
use App\Http\Controllers\Controller;
use App\Traits\ApiResponser;
use App\Wall;
use Illuminate\Http\Request;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;

class CampaignWallImpactController extends Controller
{
    use ApiResponser;

    /**
     *      Mostrar listado de audiencias disponibles para una campaña
     *
     *          Todos
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
	public function index(Campaign $campaign)
    {
    	$impacts = $campaign->walls()
    						->select('i.*')
    						->join('impacts_data as iD','iD.wall_id','=','walls.id')
                            ->join('impacts as i','i.id','=','iD.impact_id')
                            ->orderBy('i.id', 'desc')
                            ->get()
                            ->unique('i.id');
        return $this->showAll($impacts);
    }

    /**
     *      Mostrar audiencia de una campaña
     *
     *          Todos
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Campaign $campaign)
    {
        $rules = [
            'date_range'  =>  ['required', 'string']
        ];

        $this->validate(request(), $rules);

        // $impacts = Impact::where('name', request('impact'))->get();
        $data = [
        	'female_sex'	=> 0,
			'male_sex' 		=> 0,
			'ages_13_17' 	=> 0,
			'ages_18_25' 	=> 0,
			'ages_26_40' 	=> 0,
			'ages_41_55' 	=> 0,
			'ages_55' 		=> 0,
			'sl_a' 			=> 0,
			'sl_b' 			=> 0,
			'sl_c' 			=> 0,
			'sl_d' 			=> 0,
			'sl_e' 			=> 0,
			'total' 		=> 0,
			'hits'			=> 0,
            'walls'         => []
        ];

    	$impacts_data = $campaign->walls()
    						->select('iD.*')
    						->join('impacts_data as iD','iD.wall_id','=','walls.id')
                            ->join('impacts as i','i.id','=','iD.impact_id')
                            ->where('i.date_range', request('date_range'))
                            ->get();

        foreach ($impacts_data as $key => $impact) {
        	$data['female_sex'] += $impact['female_sex'];
        	$data['male_sex'] += $impact['male_sex'];
        	$data['ages_13_17'] += $impact['ages_13_17'];
        	$data['ages_18_25'] += $impact['ages_18_25'];
        	$data['ages_26_40'] += $impact['ages_26_40'];
        	$data['ages_41_55'] += $impact['ages_41_55'];
        	$data['ages_55'] += $impact['ages_55'];
        	$data['sl_a'] += $impact['sl_a'];
        	$data['sl_b'] += $impact['sl_b'];
        	$data['sl_c'] += $impact['sl_c'];
        	$data['sl_d'] += $impact['sl_d'];
        	$data['sl_e'] += $impact['sl_e'];
        	$data['total'] += $impact['total'];
        	$data['hits'] += ($impact['frequency'] * $impact['total']);
            array_push($data['walls'], $impact['pivot']['wall_id']);
        }

        return $this->showOne($data);
    }

    /**
     *      Mostrar audiencia de una pantalla
     *
     *          Todos
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function showWall(Wall $wall)
    {
        $rules = [
            'date_range'  =>  ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $impacts_wall = $wall->select('iD.*')
                            ->join('impacts_data as iD','iD.wall_id','=','walls.id')
                            ->join('impacts as i','i.id','=','iD.impact_id')
                            ->where(['walls.id' => $wall->id, 'i.date_range' => request('date_range')])
                            ->orderBy('iD.id', 'desc')
                            ->first();

        return $this->showOne($impacts_wall);
    }

    /**
    *
    ** Superadministrador
    ** Administrador
    ** Sales manager
    ** Sellers Costs
    ** Seller
    ** Customer Costs
    ** Customer
    *
    * @param  \App\Campaign  $campaign
    * @return \Illuminate\Http\Response
    */
    public function exportAudience(Campaign $campaign)
    {
        $rules = [
            'date_range'  =>  ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $walls_campaign = $campaign->walls()
                            ->select('iD.*', 'walls.provider_wall_id')
                            ->join('impacts_data as iD','iD.wall_id','=','walls.id')
                            ->join('impacts as i','i.id','=','iD.impact_id')
                            ->where('i.date_range', request('date_range'))
                            ->get();

        $spreadsheet = new Spreadsheet();

        $spreadsheet->getActiveSheet()->setTitle('Audiencia');

        $spreadsheet->getActiveSheet()->getColumnDimension('A')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('B')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('C')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('D')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('E')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('F')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('G')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('H')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('I')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('J')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('K')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('L')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('M')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('N')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('O')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('P')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('Q')->setAutoSize(true);

        $sheet = $spreadsheet->getActiveSheet();
        $sheet->setCellValue('A1', 'ID');
        $sheet->setCellValue('B1', 'Rango de fechas');
        $sheet->setCellValue('C1', 'Masculino');
        $sheet->setCellValue('D1', 'Femenino');
        $sheet->setCellValue('E1', 'Edad 13-17');
        $sheet->setCellValue('F1', 'Edad 18-25');
        $sheet->setCellValue('G1', 'Edad 26-40');
        $sheet->setCellValue('H1', 'Edad 41-55');
        $sheet->setCellValue('I1', 'Edad +55');
        $sheet->setCellValue('J1', 'Nivel socioeconomico A');
        $sheet->setCellValue('K1', 'Nivel socioeconomico B');
        $sheet->setCellValue('L1', 'Nivel socioeconomico C');
        $sheet->setCellValue('M1', 'Nivel socioeconomico D');
        $sheet->setCellValue('N1', 'Nivel socioeconomico E');
        $sheet->setCellValue('O1', 'Total');
        $sheet->setCellValue('P1', 'Frecuencia');
        $sheet->setCellValue('Q1', 'Hits');

        $idx = 2;

        foreach ($walls_campaign as $wall) {
            $sheet->setCellValue('A'.$idx, $wall->provider_wall_id);
            $sheet->setCellValue('B'.$idx, request('date_range'));
            $sheet->setCellValue('C'.$idx, number_format($wall->male_sex));
            $sheet->setCellValue('D'.$idx, number_format($wall->female_sex));
            $sheet->setCellValue('E'.$idx, number_format($wall->ages_13_17));
            $sheet->setCellValue('F'.$idx, number_format($wall->ages_18_25));
            $sheet->setCellValue('G'.$idx, number_format($wall->ages_26_40));
            $sheet->setCellValue('H'.$idx, number_format($wall->ages_41_55));
            $sheet->setCellValue('I'.$idx, number_format($wall->ages_55));
            $sheet->setCellValue('J'.$idx, number_format($wall->sl_a));
            $sheet->setCellValue('K'.$idx, number_format($wall->sl_b));
            $sheet->setCellValue('L'.$idx, number_format($wall->sl_c));
            $sheet->setCellValue('M'.$idx, number_format($wall->sl_d));
            $sheet->setCellValue('N'.$idx, number_format($wall->sl_e));
            $sheet->setCellValue('O'.$idx, number_format($wall->total));
            $sheet->setCellValue('P'.$idx, number_format($wall->frequency));
            $sheet->setCellValue('Q'.$idx, number_format(($wall->total * $wall->frequency)));
            $idx++;
        }

        $writer = new Xlsx($spreadsheet);
        $name_excel = $campaign->id.'-campana-audiencia.xlsx';
        $writer->save(public_path().'/excels/temporary/'.$name_excel);
        return $this->showMessage( $name_excel, 200 );
    }
}
