<?php

namespace App\Http\Controllers\AvailabilityWall;

use App\AvailabilityWall;
use App\Campaign;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\Wall;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;

class AvailabilityWallCampaignController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware( 'can:availability.show' )->only( 'show' );
        $this->middleware( 'can:availability.update' )->only( 'update' );
        $this->middleware( 'can:availability.indexCampaignWalls' )->only( 'indexCampaignWalls' );
    }

    /**
     * Display the specified resource.
     * @param  \App\AvailabilityWall  $availabilityWall
     * @return \Illuminate\Http\Response
    Obtener todos los muros con sus fechas seleccionadas de una campaña
     ** Superadministrators
     ** Administrators
     ** Salesmanager
     ** Sellercost
     ** Customercost
     */
    public function indexCampaignWalls(Campaign $campaign)
    {
        $availability = AvailabilityWall::where(
            [
                'campaign_id' => $campaign->id
            ])->get();

        return $this->showAll( Collection::make($availability) );
    }

    /**
     * Display the specified resource.
     * @param  \App\AvailabilityWall  $availabilityWall
     * @return \Illuminate\Http\Response
        Obtener la disponibilidad de un muro con la disponibilidad ya agregada a una campaña
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Sellercost
        ** Customercost
    */
    public function show(Wall $wall, Campaign $campaign)
    {
        if ( $campaign->editable == Campaign::NOT_EDITABLE ) {
            if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {
                return $this->errorResponse('Non-editable resource', 409);
            }
        }


        if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {

            if ( auth()->user()->hasRole( 'salesmanager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('Error, you do not have access to this resource', 403);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        }

        $availability = AvailabilityWall::where(
                            [
                                'campaign_id' => $campaign->id,
                                'wall_id' => $wall->id
                            ])->get();

        return $this->showAll( Collection::make($availability) );
    }

    /**
     * Update the specified resource in storage.
     *  Actualizar disponibilidad a una campaña con diferentes fehcas
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Sellercost
        ** Customercost
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\AvailabilityWall  $availabilityWall
     * @return \Illuminate\Http\Response
     */
    public function update(Wall $wall, Campaign $campaign)
    {
        if ( $campaign->editable == Campaign::NOT_EDITABLE ) {
            if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {
                return $this->errorResponse('Non-editable resource', 409);
            }
        }


        if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {

            if ( auth()->user()->hasRole( 'salesmanager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('Error, you do not have access to this resource', 403);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        }


        //date: [
        //  {2021: ['ft_jun']}
        //]


        $wallsAvailability = AvailabilityWall::where([
                    'wall_id'       =>  $wall->id,
                    'campaign_id'    =>  $campaign->id
                ])->get();

        if ($wallsAvailability) {
            foreach ($wallsAvailability as $key => $wallA) {
                $wallA->delete();
            }
        }

        foreach (request('dates') as $key => $dates) {
            if ($dates) {
                $weeks = [
                    'campaign_id'   =>  $campaign->id,
                    'wall_id'       =>  $wall->id
                ];
                $weeks['year'] = $key;
                foreach ($dates as $key => $date) {
                    $weeks[$date] = true;
                }
                AvailabilityWall::create($weeks);
            }
        }

        return $this->showMessage('Updated Availability', 200);
    }
}
