<?php

namespace App\Http\Controllers\Campaign;

use App\Campaign;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\MappingFile;
use Illuminate\Http\Request;

class CampaignMappingFileController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:campaigns.mapping.files.index')->only('index');
        $this->middleware('can:campaigns.mapping.files.store')->only('store');
    }

    /**
     * Listar poligonos de una campaña
     *
     * @return \Illuminate\Http\Response
    */
    public function index(Campaign $campaign)
    {
        $mapping_files = $campaign->mappingFiles;
        return $this->showAll($mapping_files);
    }

    /**
     * Agregar puntos de interes a una campaña
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Campaign $campaign)
    {

        if ( !auth()->user()->hasRole( 'superAdministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {

            if ( auth()->user()->hasRole( 'salesManager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('No tienes permiso de esta campaña', 422);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('No tienes permiso de esta campaña', 422);
                }
            }

        }

        $rules = [
            'mappingFiles.*' => ['numeric']
        ];

        $this->validate(request(), $rules);

        foreach (request( 'mappingFiles' ) as $key => $mapping_file_id) {
            if (MappingFile::find($mapping_file_id)) {
                $campaign->mappingFiles()->syncWithoutDetaching($mapping_file_id);
            }
        }

        return $this->showMessage('Added Mapping Files', 200);
    }
}
