<?php

namespace App\Http\Controllers\Campaign;

use App\Campaign;
use App\Polygon;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ApiController;

class CampaignPolygonController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:campaigns.polygons.index')->only('index');
        $this->middleware('can:campaigns.polygons.store')->only('store');
        $this->middleware('can:campaigns.polygons.destroy')->only('destroy');
    }

    /**
     * Listar poligonos de una campaña
     *
     * @return \Illuminate\Http\Response
    */
    public function index(Campaign $campaign)
    {
        $polygons = $campaign->polygons;
        return $this->showAll($polygons);
    }

    /**
     * Guardar poligonos en una campaña.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Campaign $campaign)
    {

        if ( $campaign->editable == Campaign::NOT_EDITABLE ) {
            if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrador' ) ) {
                return $this->errorResponse('Non-editable resource', 409);
            }
        }


        if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrador' ) ) {

            if ( auth()->user()->hasRole( 'salesmanager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('Error, you do not have access to this resource', 403);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        }

        // if ( !auth()->user()->hasRole('superadministrator') && !auth()->user()->hasRole('administrador') ) {
        //     if (auth()->user()->hasRole('salesmanager')) {

        //     }
        //     if ($campaign->user_registry_id != auth()->user()->id) {
        //         return $this->errorResponse('Error, you do not have access to this resource', 403);
        //     }
        // }

        $rules = [
            'polygons' => ['required']
        ];

        $this->validate(request(), $rules);

        $polygons_array_decode = json_decode(request('polygons'), true);

        if (!is_array($polygons_array_decode)) {
            return $this->errorResponse('Error, invalid polygons', 406);
        }

        $campaign->polygons()->delete();

        foreach ($polygons_array_decode as $key => $polygon_data) {
            $polygon_json = json_encode($polygon_data);
            if ($polygon_json) {
                $polygon = Polygon::create([
                    'campaign_id' => $campaign->id,
                    'polygon'  => $polygon_json
                ]);
            }
        }

        return $this->showMessage('Resources added correctly', 200);
    }

    /**
     * Eliminar un poligono de una campaña
     *
     * @param  \App\Polygon  $polygon
     * @return \Illuminate\Http\Response
     */
    public function destroy(Campaign $campaign, Polygon $polygon)
    {
        if ( $campaign->editable == Campaign::NOT_EDITABLE ) {
            if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrador' ) ) {
                return $this->errorResponse('Non-editable resource', 409);
            }
        }


        if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrador' ) ) {

            if ( auth()->user()->hasRole( 'salesmanager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('No tienes permiso de esta campaña', 422);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('No tienes permiso de esta campaña', 422);
                }
            }

        }

        if (!auth()->user()->hasRole('superadministrator') && !auth()->user()->hasRole('administrador')) {
            if ($campaign->user_registry_id != auth()->user()->id) {
                return $this->errorResponse('No tienes permiso de esta campaña', 422);
            }
        }

        if (!$campaign->polygons()->find($polygon->id)) {
            return $this->errorResponse('El poligono no existe en la campaña', 404);
        }

        $polygon->delete();

        return $this->showAll($campaign->polygons);
    }
}
