<?php

namespace App\Http\Controllers\Campaign;

use App\AvailabilityWall;
use App\Campaign;
use App\Circuit;
use App\Company;
use App\Country;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\Provider;
use App\Reseller;
use App\Wall;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;

class CampaignWallController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:campaigns.walls.index')->only('index');
        $this->middleware('can:campaigns.walls.store')->only('store');
        $this->middleware('can:campaigns.walls.destroy')->only('destroy');
        $this->middleware('can:campaigns.walls.destroyCircuit')->only('destroyCircuit');
        $this->middleware('can:campaigns.walls.export')->only('export');
    }

    /**
     * Mostrar pantallas de una campaña
        ** Superadministrador
        ** Administrador
        ** Sales manager
        ** Sellers Costs
        ** Seller
        ** Customer Costs
        ** Customer
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Campaign $campaign)
    {
        $walls = $campaign->walls;
        return $this->showAll($walls);
    }

    /**
     * Agregar pantallas a unca campaña
        ** Superadministrador
        ** Administrador
        ** Sales manager
        ** Sellers Costs
        ** Seller
        ** Customer Costs
        ** Customer
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Campaign $campaign)
    {

        if ( $campaign->editable == Campaign::NOT_EDITABLE ) {
            if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {
                return $this->errorResponse('Non-editable resource', 409);
            }
        }


        if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {

            if ( auth()->user()->hasRole( 'salesmanager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('Error, you do not have access to this resource', 403);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        }

        // {"walls_ids":[
        //         {
        //             wallId: 12,
        //             date: [
        //                {2021: ['ft_jun']}
        //             ]
        //         }
        //     ]
        // }
        // $rules = [
        //     'walls.*' => ['numeric']
        // ];

        // $this->validate(request(), $rules);


        foreach (request( 'walls' ) as $key => $data) {

            if (Wall::find($data['wallId'])) {

                $wallsAvailability = AvailabilityWall::where([
                    'wall_id'       =>  $data['wallId'],
                    'campaign_id'    =>  $campaign->id
                ])->get();

                if ($wallsAvailability) {
                    foreach ($wallsAvailability as $key => $wallA) {
                        $wallA->delete();
                    }
                }

                if (array_key_exists('dates', $data)) {
                    foreach ($data['dates'] as $key => $dates) {
                        $weeks = [];
                        foreach ($dates as $key2 => $date) {
                            $weeks['campaign_id'] = $campaign->id;
                            $weeks['wall_id'] = $data['wallId'];
                            $weeks['year'] = $key;
                            $weeks[$date] = true;
                        }
                        AvailabilityWall::create($weeks);
                        $campaign->walls()->syncWithoutDetaching($data['wallId']);
                    }
                } else {
                    $weeks = [];
                    $weeks['campaign_id'] = $campaign->id;
                    $weeks['wall_id'] = $data['wallId'];
                    AvailabilityWall::create($weeks);
                    $campaign->walls()->syncWithoutDetaching($data['wallId']);
                }
            }
        }

        return $this->showMessage('Added Walls', 200);
    }

    /**
     * Eliminar una pantalla de una campaña
     ** Superadministrador
        ** Administrador
        ** Sales manager
        ** Sellers Costs
        ** Seller
        ** Customer Costs
        ** Customer
     *
     * @param  \App\Campaign  $campaign
     * @return \Illuminate\Http\Response
     */
    public function destroy(Campaign $campaign, Wall $wall)
    {
        if ( $campaign->editable == Campaign::NOT_EDITABLE ) {
            if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {
                return $this->errorResponse('Non-editable resource', 409);
            }
        }


        if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {

            if ( auth()->user()->hasRole( 'salesmanager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('Error, you do not have access to this resource', 403);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        }

        if (!$campaign->walls()->find($wall->id)) {
            return $this->errorResponse('The wall does not exist in the campaign', 404);
        }

        $wallsAvailability = AvailabilityWall::where([
            'wall_id'       =>  $wall->id,
            'campaign_id'    =>  $campaign->id
        ])->get();

        if ($wallsAvailability) {
            foreach ($wallsAvailability as $key => $wallA) {
                $wallA->delete();
            }
        }

        $campaign->walls()->detach($wall->id);
        return $this->showAll($campaign->walls);
    }

    /**
     * Eliminar una pantallas de un circuito a una campaña
     ** Superadministrador
        ** Administrador
        ** Sales manager
        ** Sellers Costs
        ** Seller
        ** Customer Costs
        ** Customer
     *
     * @param  \App\Campaign  $campaign
     * @return \Illuminate\Http\Response
     */
    public function destroyCircuit(Campaign $campaign, Circuit $circuit)
    {
        if ( $campaign->editable == Campaign::NOT_EDITABLE ) {
            if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {
                return $this->errorResponse('Non-editable resource', 409);
            }
        }


        if ( !auth()->user()->hasRole( 'superadministrator' ) && !auth()->user()->hasRole( 'administrator' ) ) {

            if ( auth()->user()->hasRole( 'salesmanager' ) ) {

                if ( auth()->user()->id != $campaign->user_registry_id ) {

                    if ( $campaign->seller->admin_registry_id != auth()->user()->id ) {
                        return $this->errorResponse('Error, you do not have access to this resource', 403);
                    }

                }

            } else {
                if (auth()->user()->id != $campaign->user_registry_id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        }

        $wallsAvailability = AvailabilityWall::where('campaign_id', $campaign->id)
                                            ->whereIn('wall_id', $circuit->walls->pluck('id'))
                                            ->get();

        if ($wallsAvailability) {
            foreach ($wallsAvailability as $key => $wallA) {
                $wallA->delete();
            }
        }

        foreach ($circuit->walls as $key => $wall) {
            $campaign->walls()->detach($wall->id);
        }

        return $this->showAll($campaign->walls);
    }

    /**
    * Eliminar una pantallas de un circuito a una campaña
    ** Superadministrador
    ** Administrador
    ** Sales manager
    ** Sellers Costs
    ** Seller
    ** Customer Costs
    ** Customer
    *
    * @param  \App\Campaign  $campaign
    * @return \Illuminate\Http\Response
    */
    public function export(Campaign $campaign)
    {

        $walls_campaign = $campaign->walls;

        $spreadsheet = new Spreadsheet();

        $spreadsheet->getActiveSheet()->setTitle('OOH');

        $isAdmin = false;

        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
            $isAdmin = true;
        }

        $companiesIds = [];
        foreach ($walls_campaign as $key => $wall) {
            if (!in_array($wall->company_id, $companiesIds)) {
                array_push($companiesIds, $wall->company_id);
            }
        }

        $companiesMultiplier = Provider::whereIn( 'company_id', $companiesIds )->get();

        $multiplierReseller = Reseller::where('company_id', $campaign->seller->company_id)->first();
        $multiplierReseller = $multiplierReseller ? $multiplierReseller->multiplier : 1;

        $spreadsheet->getActiveSheet()->getColumnDimension('A')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('B')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('C')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('D')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('E')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('F')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('G')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('H')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('I')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('J')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('K')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('L')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('M')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('N')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('O')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('P')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('Q')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('R')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('S')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('T')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('U')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('V')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('W')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('X')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('Y')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('Z')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AA')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AB')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AC')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AD')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AE')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AF')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AG')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AH')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AI')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AJ')->setAutoSize(true);
        $spreadsheet->getActiveSheet()->getColumnDimension('AK')->setAutoSize(true);

        $sheet = $spreadsheet->getActiveSheet();
        $sheet->setCellValue('A1', 'ID');
        $sheet->setCellValue('B1', ($isAdmin) ? 'proveedor' : '***');
        $sheet->setCellValue('C1', 'Nombre');
        $sheet->setCellValue('D1', 'Pais');
        $sheet->setCellValue('E1', 'Estado');
        $sheet->setCellValue('F1', 'Municipio/Alcaldia');
        $sheet->setCellValue('G1', 'Colonia');
        $sheet->setCellValue('H1', 'Calle');
        $sheet->setCellValue('I1', 'Numero');
        $sheet->setCellValue('J1', 'Codigo postal');
        $sheet->setCellValue('K1', 'Ooh');
        $sheet->setCellValue('L1', 'Tipo pantalla');
        $sheet->setCellValue('M1', 'Formato');
        $sheet->setCellValue('N1', 'Medida metros');
        $sheet->setCellValue('O1', 'Impactos promedio mensual');
        $sheet->setCellValue('P1', 'Velocidad promedio');
        $sheet->setCellValue('Q1', 'Tipo de negocio');
        $sheet->setCellValue('R1', 'Nombre de negocio');
        $sheet->setCellValue('S1', 'Numero de caras');
        $sheet->setCellValue('T1', 'Descripcion');
        $sheet->setCellValue('U1', 'Ubicacion (Google maps)');
        $sheet->setCellValue('V1', 'Latitud');
        $sheet->setCellValue('W1', 'Longitud');
        $sheet->setCellValue('X1', 'Material del arte');
        $sheet->setCellValue('Y1', 'Modelo de venta');
        $sheet->setCellValue('Z1', 'Impresión por terceros');
        $sheet->setCellValue('AA1','Tipo de formato');
        $sheet->setCellValue('AB1','Calidad de arte');
        $sheet->setCellValue('AC1','Requerimientos adicionales');
        $sheet->setCellValue('AD1', ($isAdmin) ? 'Costo Moneda Local' : ' ');
        $sheet->setCellValue('AE1', ($isAdmin) ? 'Tarifa Moneda Local' : ' ');
        $sheet->setCellValue('AF1', ($isAdmin) ? 'Costo DLL' : ' ');
        $sheet->setCellValue('AG1', ($isAdmin) ? 'Tarifa DLL' : ' ');
        $sheet->setCellValue('AH1', ($isAdmin) ? 'Costo Producción Moneda Local' : ' ');
        $sheet->setCellValue('AI1', ($isAdmin) ? 'Costo Producción DLL' : ' ');
        $sheet->setCellValue('AJ1', ($isAdmin) ? 'Costo de instalación Moneda Local' : ' ');
        $sheet->setCellValue('AK1', ($isAdmin) ? 'Costo de instalación DLL' : ' ');


        $idx = 2;
        foreach ($walls_campaign as $wall) {
            $company = '';
            if ($isAdmin) {
                $company = Company::where('id', $wall->company_id )->first();
            }

            $country = Country::where('id', $wall->country_id)->first();

            $sheet->setCellValue('A'.$idx, $wall->id);
            $sheet->setCellValue('B'.$idx, ($isAdmin) ? $company->name_company : '***');
            $sheet->setCellValue('C'.$idx, $wall->provider_wall_id);
            $sheet->setCellValue('D'.$idx, $country->country_name);
            $sheet->setCellValue('E'.$idx, $wall->state);
            $sheet->setCellValue('F'.$idx, $wall->town_hall_delegation);
            $sheet->setCellValue('G'.$idx, $wall->suburb);
            $sheet->setCellValue('H'.$idx, $wall->street);
            $sheet->setCellValue('I'.$idx, $wall->location_number);
            $sheet->setCellValue('J'.$idx, $wall->postal_code);
            $sheet->setCellValue('K'.$idx, $wall->ooh);
            $sheet->setCellValue('L'.$idx, $wall->type);
            $sheet->setCellValue('M'.$idx, $wall->vertical_horizontal);
            $sheet->setCellValue('N'.$idx, $wall->measured_in_meters);
            $sheet->setCellValue('O'.$idx, $wall->average_monthly_impacts);
            $sheet->setCellValue('P'.$idx, $wall->average_speed);
            $sheet->setCellValue('Q'.$idx, $wall->business_type);
            $sheet->setCellValue('R'.$idx, $wall->business_name);
            $sheet->setCellValue('S'.$idx, $wall->number_faces);
            $sheet->setCellValue('T'.$idx, $wall->description);
            $sheet->setCellValue('U'.$idx, 'Ubicación pantalla');
            $sheet->getCell('U'.$idx)->getHyperlink()->setUrl('https://maps.google.com/?q='.$wall->latitude.','.$wall->longitude);
            $sheet->getStyle('U'.$idx)->applyFromArray(array( 'font' => array( 'color' => ['rgb' => '0000FF'], 'underline' => 'single' ) ));
            $sheet->setCellValue('V'.$idx, $wall->latitude);
            $sheet->setCellValue('W'.$idx, $wall->longitude);

            $salesModel = '';

            switch ($wall->sales_model) {
                case '1':
                    $salesModel = 'Semanal';
                    break;
                case '2':
                    $salesModel = 'Quincenal';
                    break;
                case '4':
                    $salesModel = 'Mensual';
                    break;
                default:
                    $salesModel = '';
                    break;
            }

            $sheet->setCellValue('X'.$idx, $wall->material_art);
            $sheet->setCellValue('Y'.$idx, $salesModel);
            $sheet->setCellValue('Z'.$idx, $wall->third_party_printing ? 'Si' : 'No');
            $sheet->setCellValue('AA'.$idx, $wall->art_format);
            $sheet->setCellValue('AB'.$idx, $wall->quality_art);
            $sheet->setCellValue('AC'.$idx, $wall->additional_requirements);

            $cost = 0;

            foreach ($companiesMultiplier as $key => $company) {
                if ($company->company_id == $wall->company_id) {
                    $multiplier = $company->multiplier ? $company->multiplier : 1;
                    $cost = $wall->local_currency *  $multiplier;
                }
            }

            $cost = $cost != 0 ? $cost : $wall->local_currency;

            $sheet->setCellValue('AD'.$idx, ($isAdmin) ? $wall->local_currency : '');
            $sheet->setCellValue('AE'.$idx, ($isAdmin) ? $cost : '');
            $sheet->setCellValue('AF'.$idx, ($isAdmin) ? $wall->local_currency / $country->dollar_change : '');
            $sheet->setCellValue('AG'.$idx, ($isAdmin) ? $cost / $country->dollar_change : '');

            $sheet->setCellValue('AH'.$idx, ($isAdmin) ? $wall->production_cost : '');
            $sheet->setCellValue('AI'.$idx, ($isAdmin) ? $wall->production_cost / $country->dollar_change : '');

            $sheet->setCellValue('AJ'.$idx, ($isAdmin) ? $wall->installation_cost : '');
            $sheet->setCellValue('AK'.$idx, ($isAdmin) ? $wall->installation_cost / $country->dollar_change : '');

            $idx++;
        }

        $writer = new Xlsx($spreadsheet);

        $name_excel = $campaign->id.'-campana.xlsx';

        $writer->save(public_path().'/excels/temporary/'.$name_excel);

        return $this->showMessage( $name_excel, 200 );

    }

    // Validar
    public function getCampaignsWallAvailability(Wall $wall)
    {
        $campaigns = Campaign::select('campaigns.id', 'campaigns.name', 'u.name as name_seller', 'u.surnames as surnames_seller', 'uv.name as name_customer', 'uv.surnames as surnames_customer')
                            ->join('availability_walls as aw','aw.campaign_id','=','campaigns.id')
                            ->join('users as u','u.id','=','campaigns.user_registry_id')
                            ->join('users as uv','uv.id','=','campaigns.customer_campaign_id')
                            ->where([
                                'aw.wall_id' => $wall->id,
                                'aw'.".".request('week') => true,
                                'aw.year' => request('year')
                            ])
                            ->get();

        return $this->showAll(Collection::make($campaigns), 201);
    }
}
