<?php

namespace App\Http\Controllers\Circuit;

use App\Circuit;
use App\Http\Controllers\ApiController;
use Illuminate\Database\Eloquent\Collection;
use App\Http\Controllers\Controller;

class CircuitController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:circuits.store')->only('store');
        $this->middleware('can:circuits.update')->only('update');
    }

    /**
     * Metodo para listar todos los circuitos
        ROLES
        ** Todos los roles
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        if (request()->get('per_page') === 'all') {
            return $this->showAll( Circuit::all() );
        }
        $circuits = Circuit::orderBy( 'id', 'desc' )
                            ->withCount('walls')
                            ->paginate(8);
        return $this->showAll(Collection::make($circuits));
    }

    /**
     * Metodo para listar todos los circuitos
        ROLES
        ** Superadministradores
        ** Administradores
     *
     * @return \Illuminate\Http\Response
     */
    public function store()
    {
        $rules = [
            'name' => ['required', 'string', 'max:255'],
        ];
        $this->validate(request(), $rules);
        $circuit = Circuit::create(request()->all());
        return $this->showOne($circuit, 201);
    }

    /**
     * Metodo para listar todos los circuitos
        ROLES
        ** Todos los roles
     *
     * @return \Illuminate\Http\Response
     */
    public function show(Circuit $circuit)
    {
        return $this->showOne($circuit);
    }

    /**
     * Metodo para listar todos los circuitos
        ROLES
        ** Superadministradores
        ** Administradors
     *
     * @return \Illuminate\Http\Response
     */
    public function update(Circuit $circuit)
    {
        $rules = [
            'name' => ['string']
        ];

        $this->validate(request(), $rules);

        if (request('name')) {
            $circuit->name = request('name');
        }

        if($circuit->isClean()) {
            return $this->errorResponse('No se encontraron cambios', 422);
        }

        $circuit->save();

        return $this->showOne($circuit);
    }
}
