<?php

namespace App\Http\Controllers\Country;

use App\Country;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\PublicTransport;
use App\State;
use App\Wall;

class CountryController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:countries.store')->only('store');
        $this->middleware('can:countries.update')->only('update');
        // $this->middleware('can:countries.destroy')->only('destroy');
    }

    /**
     * Metodo para listar todos los paises
        ROLES
        ** Todos los roles
     *
     * @return \Illuminate\Http\Response
    */
    public function index()
    {
        $countries = [];
        if (count(auth()->user()->company->countries) > 0) {
            $countries = auth()->user()->company->countries;
        } else {
            $countries = Country::orderBy( 'country_name', 'asc' )->get();
        }
        return $this->showAll($countries);
    }

    /**
     * Metodo para listar todos los paises pantallas tipo tradicional
        ROLES
        ** Todos los roles
     *
     * @return \Illuminate\Http\Response
    */
    public function getCountriesTraditional()
    {
        $countries = [];
        if (count(auth()->user()->company->countries) > 0) {
            $countries = auth()->user()->company->countries;
        } else {
            $countries = Wall::where('programatic_traditional', 'traditional')
                                 ->with('country')
                                 ->get()
                                 ->pluck('country')
                                 ->unique()
                                 ->sortBy('country_name')
                                 ->filter()
                                 ->values();
        }
        return $this->showAll($countries);
    }

    /**
     * Metodo para listar todos los paises que tengan transporte publico
        ROLES
        ** Todos los roles
     *
     * @return \Illuminate\Http\Response
    */
    public function getCountriesPublicTransport()
    {
        $countries = [];
        if (count(auth()->user()->company->countries) > 0) {
            $countries = auth()->user()->company->countries;
        } else {
            $countries = PublicTransport::with('country')
                                 ->get()
                                 ->pluck('country')
                                 ->unique()
                                 ->sortBy('country_name')
                                 ->filter()
                                 ->values();
        }
        return $this->showAll($countries);
    }

    /**
     * Metodo para listar todos estados de un arreglo de paises
        ROLES
        ** Todos los roles
     *
     * @return \Illuminate\Http\Response
    */
    public function getStatesCountry() {

         $rules = [
                    'countries_ids.*' => ['numeric']
                ];

        $this->validate(request(), $rules);

        $states = State::whereIn('country_id', request('countries_ids'))->get();

        return $states;
    }

    /**
     * Metodo para listar todos estados de un arreglo de paises existentes en directorio transporte publico
        ROLES
        ** Todos los roles
     *
     * @return \Illuminate\Http\Response
    */
    public function getStatesCountryPublicTransport() {
         $rules = [
                    'countries_ids.*' => ['numeric']
                ];

        $this->validate(request(), $rules);

        $states = PublicTransport::select('state')
                                ->whereIn('country_id', request('countries_ids'))
                                ->groupBy('state')
                                ->get();

        return $states;
    }

    /**
     * Metodo crear nuevos paises
        ROLES
        ** Superadministrador
        ** Administrador
     *
     * @return \Illuminate\Http\Response
    */
    public function store()
    {
        $rules = [
            'country_name' => ['required', 'string', 'max:255'],
            'dollar_change' => ['required', 'numeric'],
            'tax' => ['required', 'numeric'],
            'type_of_currency' => ['required', 'string', 'max:100']
        ];
        $this->validate(request(), $rules);
        $country = Country::create(request()->all());
        return $this->showOne($country, 201);
    }

    /**
     * Metodo mostrar un pais
        ROLES
        ** Todos los roles
     *
     * @return \Illuminate\Http\Response
    */
    public function show(Country $country)
    {
        return $this->showOne($country);
    }

    /**
     * Metodo para actualizar paises
        ROLES
        ** Superadministrador
        ** Administrador
     *
     * @return \Illuminate\Http\Response
    */
    public function update(Country $country)
    {
        $rules = [
            'country_name' => ['string', 'max:255'],
            'dollar_change' => ['numeric'],
            'tax' => ['numeric'],
            'type_of_currency' => ['string', 'max:100']
        ];

        $this->validate(request(), $rules);

        $country->fill(request()->only(
            'country_name', 'dollar_change', 'tax', 'type_of_currency'
        ));

        $country->save();

        return $this->showOne($country);
    }

    /**
     * Metodo para elimiar paises
        ROLES
        ** Superadministrador
     *
     * @return \Illuminate\Http\Response
    */
    // public function destroy(Country $country)
    // {
        // $country->delete();
        // return $this->showOne($country);
    // }
}
