<?php

namespace App\Http\Controllers\File;

use App\File;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class FileController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:screens.store')->only('store');
        $this->middleware('can:screens.destroy')->only('destroy');
        $this->middleware('can:screens.uploadImage')->only('uploadImage');
        $this->middleware('can:screens.deleteImage')->only('deleteImage');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $files = File::all();
        return $this->showAll( $files );
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store()
    {
        $rules = [
            'name' => ['required', 'string'],
            'version' => ['required', 'string'],
            'description' => ['string'],
            'file' => ['required', 'string'],
        ];
        $this->validate(request(), $rules);
        $file = File::create(request()->all());
        return $this->showOne($file, 201);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\File  $file
     * @return \Illuminate\Http\Response
     */
    public function destroy(File $fileManager)
    {
        Storage::disk('file-manager')->delete( $fileManager->file );
        $fileManager->delete();
        return $this->showOne($fileManager);
    }

    /**
     * Subir archivo
        ** Superadministrador
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function uploadFile() {
        $file = request()->file( 'file' );
        $input = array( 'file' => $file );
        $rules = [ 'file' => ['required'] ];
        $validate = Validator::make($input, $rules);
        if ($validate->fails())
        {
            return $this->errorResponse('File is required', 422);
        }
        $data['file'] = request('file')->store('', 'file-manager');
        return $this->responseUploadFile( $data, 201 );
    }

    /**
     * Eliminar archivo
        ** Superadministrador
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deleteFile() {
        $rules = [
            'file'        =>    ['required', 'string']
        ];

        $this->validate( request(), $rules );

        Storage::disk('file-manager')->delete( request( 'file' ) );

        return $this->showMessage('Deleted File', 200);
    }
}
