<?php

namespace App\Http\Controllers\Tag;

use Illuminate\Database\Eloquent\Collection;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\Tag;

class TagController extends ApiController
{

    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:tags.store')->only('store');
        $this->middleware('can:tags.update')->only('update');
        $this->middleware('can:tags.destroy')->only('destroy');
    }

    /**
     * Metodo para listar todos los tags
        ROLES
        ** Todos los roles
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        if (request()->get('per_page') === 'all') {
            return $this->showAll(Tag::all());
        }
        $tags = Tag::orderBy( 'id', 'desc' )->paginate(8);
        return $this->showAll(Collection::make($tags));
    }

   /**
     * Crear tags
        ROLES
        ** Superadministradores
        ** Administradores
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store()
    {
        $rules = [
            'title_tag' => ['required', 'string', 'max:255'],
        ];
        $this->validate(request(), $rules);
        $tag = Tag::create(request()->all());
        return $this->showOne($tag, 201);
    }

    /**
     * Mostrar tag
        ROLES
        ** Todos los roles
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function show(Tag $tag)
    {
        return $this->showOne($tag);
    }

    /**
     * Actualizar tags
        ROLES
        ** Superadministradores
        ** Administradores
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function update(Tag $tag)
    {
        $rules = [
            'title_tag' => ['string']
        ];

        $this->validate(request(), $rules);

        if (request('title_tag')) {
            $tag->title_tag = request('title_tag');
        }

        if($tag->isClean()) {
            return $this->errorResponse('No se encontraron cambios', 422);
        }

        $tag->save();

        return $this->showOne($tag);
    }

    /**
     * Eliminar tags
        ROLES
        ** Superadministradores
        ** Administradores
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function destroy(Tag $tag)
    {
        $tag->delete();
        return $this->showOne($tag);
    }
}
