<?php

namespace App\Http\Controllers\Team;

use App\Country;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\Team;
use Illuminate\Http\Request;

class TeamCountryController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware( 'can:teams.countries.store' )->only( 'store' );
        $this->middleware( 'can:teams.countries.show' )->only( 'show' );
        $this->middleware( 'can:teams.countries.destroy' )->only( 'destroy' );
    }

    /**
     * Metodo para agregar paises a un equipo
        ROLES
        ** Superadministrators
     *
     * @return \Illuminate\Http\Response
     */
    public function store(Team $team)
    {
        // $team->countries()->syncWithoutDetaching( $country->id );
        // return $this->showOne( $team );

        $rules = [
            'countries_ids.*' => ['numeric']
        ];

        $this->validate(request(), $rules);

        $team->countries()->detach();

        foreach (request( 'countries_ids' ) as $key => $country_id) {
            $team->countries()->syncWithoutDetaching( $country_id );
        }

        $team = $team->load('users')->load('countries');

        return $this->showOne($team);
    }

    /**
     * Metodo para mostrar los paises de un equipo
        ROLES
        ** Superadministrators
     *
     * @return \Illuminate\Http\Response
     */
    public function show(Team $team)
    {
        return $this->showAll( $team->countries()->get() );
    }

    /**
     * Metodo para eliminar un pais de un equipo
        ROLES
        ** Superadministrators
     *
     * @return \Illuminate\Http\Response
    */
    public function destroy(Team $team, Country $country)
    {
        $team->countries()->detach($country->id);
        return $this->showOne($team);
    }
}
