<?php

namespace App;

use App\Campaign;
use App\Company;
use App\Country;
use App\Team;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Passport\HasApiTokens;
use Spatie\Permission\Traits\HasRoles;

class User extends Authenticatable
{
    // Permitir a laravel autentificar usuarios, atravez de acces tokens, mediante el trait (laravel passport) de HasApiTokens
    use Notifiable, HasApiTokens, HasRoles;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id',
        'country_id',
        'user_registry_id',
        'admin_registry_id',
        'name',
        'surnames',
        'email',
        'password',
        'img_profile'
    ];

    /*  MUTADORES */
    public function setNameAttribute($name) {
        $this->attributes['name'] = mb_strtolower($name);
    }

    public function setEmailAttribute($email) {
        $this->attributes['email'] = strtolower($email);
    }

    public function setSurnamesAttribute($surnames) {
        $this->attributes['surnames'] = mb_strtolower($surnames);
    }

    public function setPasswordAttribute($value)
    {
        $this->attributes['password'] = bcrypt($value);
    }

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    public function teams() {
        return $this->belongsToMany(Team::class, 'user_team');
    }

    public function company() {
        return $this->belongsTo(Company::class);
    }

    public function country() {
        return $this->belongsTo(Country::class);
    }

    public function campaigns_sellers() {
        return $this->hasMany(Campaign::class, 'user_registry_id');
    }

    public function campaigns_customers() {
        return $this->hasMany(Campaign::class, 'customer_campaign_id');
    }

    public function customers() {
        return $this->hasMany(User::class, 'user_registry_id');
    }

    public function users_admin() {
        return $this->hasMany(User::class, 'admin_registry_id');
    }

    // Query Scopes
    public function scopeName($query, $name)
    {
        if ($name)
            return $query->where('name', 'LIKE', "%$name%");
    }

    public function scopeSurnames($query, $surnames)
    {
        if ($surnames)
            return $query->where('surnames', 'LIKE', "%$surnames%");
    }

    public function scopeEmail($query, $email)
    {
        if ($email)
            return $query->where('email', 'LIKE', "%$email%");
    }

}
